/*************************************************************************** 
*  init.c
*
*  Gergely Zaruba
*  August 2007
*  NOTE: All MSP430 Constants are defined in the c:\mspgcc\msp430\include
*        directory!
*
****************************************************************************/
#include "init.h"

void init()
{
  init_watchdog();
  init_clock();
  init_timer();
  init_ports();
  init_serial();
  
  // enable general interrupts
  eint();
  
  pause(100); // pause in milliseconds - to allow for any transients what so ever 
              //to settle
}

/************************************************************************
 Disable watchdog
 ***********************************************************************/ 
void init_watchdog()
{
    // deactivate watchdog timer for now

    WDTCTL = WDTPW + WDTHOLD;

}

/************************************************************************
 Set up Clock
 ***********************************************************************/ 
void init_clock()
{
  // system master clock (MCLK) starts using the DCO internal RC clock
  // auxiliary clock ACLK sources from external crystal 1 (32kHz)
  _BIC_SR(OSCOFF);  //turn off LFXT1
  _BIC_SR(CPUOFF);
  _BIS_SR(SCG0);
  _BIC_SR(SCG1);
  BCSCTL1 = 0; //DCO to lowest freq
  BCSCTL2 = SELM_XT2CLK + SELS; // XT2 sources MCLK and SMCLK
  while(IFG1 & OFIFG) //have to make sure XT2 is stabile
    {
      IFG1 &= ~OFIFG;
      P1IFG = 0; // do nothing
    }
}

/************************************************************************
 Set up Timer A and B
 ***********************************************************************/ 
void init_timer()
{
  // we'll use timer A for system clock and half second measurements
  // set clock source to SMclock, 8MHz
  // divided by 8, so about 1us
  // count to 0xFFFF and create interrupt TAIFG 
  TACTL = TACLR; //=4 resets TACLEAR BIT
  TACTL = TASSEL_SMCLK | MC_2 | ID_3 | TAIE; 
  
  // enable compare 0 interrupt on timer A
  TACCTL0 = CCIE;
  
  // set timer length TACCR0 to 50,000 that is 50ms
  TACCR0 = 50000;
  fifty_milli_second = false;
}

/************************************************************************
 Set up I/O ports
 ***********************************************************************/ 
void init_ports()
{

  //setting unused/unconnected ports to i/o outputs is the norm

  //port 1 is to address slave ADCs
  P1SEL = 0x00; //All P1 pins are i/o and not periferal
  P1DIR = 0xFF; //All P1 pins are output (to reduce power consumption
  P1IE = 0x00;  //All P1 interrupts disabled
  P1OUT = 0x80; //make sure none of the slaves are addressed
                //the !enable bit of the demux is on P1.7
  
  // port 2 shall be our eight output for the general purpose "LED display"
  P2SEL = 0x00; //All P2 pins are i/o and not periferal
  P2DIR = 0xFF; //All P2 pins are output (to reduce power consumption
  P2OUT = 0xFF; //All p2 pins are set high, so the LEDs will not light
  P2IES = 0xFF; //All P2 pins will use low-to-high transition to trigger irq
  P2IE = 0x00;  //All P2 interrupts disabled

  // port 3 contains USART0, USAART 1 
  P3OUT = 0x00;
  P3DIR = 0xFF;
  P3SEL = 10; //P3.1 and P3.3 are I2C/UART periferals
  P3SEL |= 192; //P3.6 and P3.7 are UART periferals
  /* P3DIR = 0xFF; //preset all pins to output
  //P3DIR &= ~32; //P3.5 is URXD0 thus input
  P3DIR &= ~128; //P3.7 is URXD1 thus input
  */

  //Port 4 is unused except for (unused) control of XBee module
  P4SEL = 0x00; //All P4 pins are i/o and not periferal
  P4DIR = 0xFF; //preset P4 to all outputs
  P4DIR &= ~4; //P4.2 is input 
  P4DIR &= ~8; //P4.3 is input
  P4OUT = 01; //do not keep reset down
  
  //port 5 is unused
  P5SEL = 0x70; //all P5 pins are set to i/o not periferal 
  P5DIR = 0xFF; //all P5 pins configured as outputs.

  //P6 is unused (8 A/D converters)
  P6SEL = 0x00; //all P6 pins are i/o
  P6DIR = 0xFF; //all P6 pins are outputs

}


/************************************************************************
 Set up UART
 ***********************************************************************/ 
void init_serial()
{
  //USART 0 is used for I2C communication betwenn the 1611 master
  //and the 2013 slaves.
  U0CTL = I2C + SYNC; // select I2C mode (make sure I2CEN is off)
                      //implicitely this will also select 7 bit adressing

  I2CTCTL = I2CSSEL_SMCLK; // SMCLK
  I2CPSC = 0;        // Do not divide SMCLK


  //The I2C Master transmitter hangs if the values below are too low
  // it is due to the slaves bein too slow. solution: slave speed up'd to 20MHz
  //since most slave work seems to be during the high period of the clock
  //it is better to use an asymetrick clock (I2CSCLH!=I2CSCLL)


  //the below will result in an asymetric ~50kbps
  I2CPSC = 0;
  I2CSCLH = 115;     
  I2CSCLL = 25; 


  I2COA = 0x7F;   //logical address of the controler

  I2CIE = 127;  // Enable all I2C interrupts except STTIE
  
  U0CTL |= I2CEN; // enable I2C


  //*************************************************************
  //*************************************************************
  //we use UART 1 for serial communications to the host system
  //as USART0 is the only of the two ports that supports i2c to be used
  //to talk to the worker a/d microcontrollers
  // hold uart 1 while we change things
  U1CTL = SWRST; 
  
  // choose 8 bits, no parity, one stop bit
  //U1CTL &= ~PENA;
  //U1CTL &= ~SPB;
  U1CTL |= CHAR; //8 bit data
  //U1CTL &= ~LISTEN;
  //U1CTL &= ~SYNC;
  //U1CTL &= ~MM;
  
  // initialize transmitter options
  
  // use main clock for timing
  U1TCTL = SSEL_SMCLK;
  //U1TCTL &= ~CKPL;
  //U1TCTL &= ~URXSE;
  
  // set 57.6kbps (use msp430 baud rate calculator) calculator
  // with 8MHz clock
  UBR01=0x8A; UBR11=0x00; UMCTL1=0xEF; // uart1 8000000Hz 57595bps
  
  // receiver drops bad packets
  U1RCTL &= ~URXEIE;
  U1RCTL = 0;
  
  // enable both transmitter and receiver modules
  ME2 |= UTXE1 | URXE1;
  
  // Place UART0's pins into module mode
  //P3SEL = 0x30;
  
  // release uart for operation
  U1CTL &= ~SWRST;
  
  // enable transmitter and receiver interrupts (has to be after SWRST)
  IE2 |= UTXIE1 | URXIE1;
}

